/***************************************************************************
                          appwindow.cpp  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id$
 *
 */

#include "qtabman.h"

ApplicationWindow::ApplicationWindow(QString conn)
    : QMainWindow( 0, "MainWindow" )
{
	/* init the vars */
	tabmanscanstate = TRUE;
	debugstate = FALSE;
	tabmanwindowstate = FALSE;
	debugwindowstate = FALSE;
	tablewindowstate = FALSE;
	indexwindowstate = FALSE;
	constraintwindowstate = FALSE;
	columnwindowstate = FALSE;

	/* Create the debugdialog */
	this->debugdialog = new DebugDialog(this,"Debugdialog");

	/* Create the tabmandialog */
	this->tabmandialog = new TabmanDialog(this,"Tabmandialog");
	
	/* Create the tabledialog */
	this->tabledialog = new TableDialog(NULL,NULL,this,"Tabledialog");
	
	/* Create the index dialog */
	/* not done yet */
		
	/* Create the constraint dialog */	
	/* not done yet */
	
	/* Create the column dialog */	
	/* not done yet */
	
	/* create the status bar */
	/* not done yet */
	
	statusbar = new StatusBar(this,"statusbar");
	
	/* create the toolbar */
	toolbar = new ToolBar(this,"toolbar");
	
	/* make a menu */
	this->menu = new MenuBar(this,"menu");
	
	/* logon to the database*/
	this->connected=FALSE;
	this->connectstring = conn;
	
	this->Logon();
	
	/* create a timer to scan the tabmanpipe */
	QTimer *timer = new QTimer( this );
    
	/* Create the central widget , a splitter */
	this->split = new QSplitter( QSplitter::Horizontal, this , "main" );
	setCentralWidget( split );

	/* create the treelist */
	this->schematree = new SchemaTree( this->schemaname,split,"Schema" ,NULL);
	
	/* create the listview at the righthandside */
	this->listview = new SchemaList(split,"Schema list");
	
	/* set the schematree property, so it kwnows where the listview is */
	this->schematree->listview = listview;

	/* Connect the debugdialog slots and signals */
	QObject::connect( debugdialog, SIGNAL(Closed()),
			         this, SLOT(HideDebugDialog()) );	

	/* Connect the tabmandialog slots and signals */
	QObject::connect( tabmandialog, SIGNAL(Closed()),
			         this, SLOT(HideTabmanDialog()) );	
	
	/* Connect the tabledialog slots and signals */
	QObject::connect( tabledialog, SIGNAL(Closed()),
			         this, SLOT(HideTableDialog()) );	
					
	QObject::connect( timer, SIGNAL(timeout()),this, SLOT(ScanTabmanPipe()) );
	
	/* Connect the treechanged signal to the tree */
	QObject::connect( this, SIGNAL(TreeChanged()),
			         schematree, SLOT(RefreshTree()) );	
	
	/* Connect the Listchanged signal to the list */
	QObject::connect( this, SIGNAL(TreeChanged()),
			          listview, SLOT(DoRefresh()) );
	
	QObject::connect( listview, SIGNAL(NewObjectSelected(QString,QString,QString,QString)),
			          this, SLOT(SetNewObjectSelection(QString,QString,QString,QString)) );
	
	QObject::connect( listview, SIGNAL(Click()),this, SLOT(ListClicked()) );
		
	/* connect the Objectcountchanged signal from the listview to the statusbar */
	QObject::connect( listview, SIGNAL(ObjectCountChanged(int)),
			          statusbar, SLOT(ShowObjectCount(int)) );	
	
	
	/* start the timer */
	timer->start( 1500, FALSE );
	
	/* size things */
	QValueList<int> list;
	
	list.append(1);
	list.append(2);
	this->split->setSizes(list);
	this->resize(500,450);
}

ApplicationWindow::~ApplicationWindow()
{
	if(this->connected)
		DisconnectFromOracle();
}

void ApplicationWindow::SetUnsetDebug()
{
	this->debugstate = !this->debugstate;
	this->menu->SetDebugState(this->debugstate);
}

void ApplicationWindow::SetUnsetTabmanMessages()
{
	this->tabmanscanstate = !this->tabmanscanstate;
	this->menu->SetTabmanState(this->tabmanscanstate);
}

void ApplicationWindow::Logon()
{
	char *err=NULL;
	bool keeptrying=TRUE;
	int t;
	int pos1;
	int pos2;
	QString db;

	/* Ignore child signals !!! */
	/* the oracle user process will terminate and we */
	/* would catch that child otherwise */
	signal(SIGCHLD,SIG_IGN);

	if(!this->connectstring.isEmpty())
	{
		/* a connectstring was given at the commandline */
		if(!ConnectToOracleByString((const char*)connectstring,&err))
		{
			printf("Error connecting : %s",err);
			exit(1);
		}
		else
		{
			/* connection made, now extract username,.. from argv[1] */
			pos1 = connectstring.find('@',0,FALSE);
			if(pos1==-1) /* default database */
			{
				this->database = QString(getenv("TWO_TASK"));
			}
			else /* extract the tnsname after the @ */
			{
				this->database = connectstring.mid(pos1+1,strlen(connectstring)-pos1);
			}
			pos2 = connectstring.find('/',0,FALSE);
			
			if(pos2==0) /* '/' logon done */
			{
				this->username = QString("/");
			}			
			else
			{
				this->username = connectstring.left(pos2);
				this->password = connectstring.mid (pos2+1,pos1-pos2-1);
			}
		}
		this->connected = TRUE;
		this->SetSchemaName();
		this->connectstring = NULL;
		this->SetNewConnectString();
		return;
	}

	LoginDialog ld((QWidget*)this,"Login");
	do
	{
		if ( ld.exec() )
		{

			if(this->connected)
					DisconnectFromOracle();

			this->connected = FALSE;

   	  /* insert '/' if username is null                */
   	  /* this is the famous Jorgen Bosman logon method */
   	  /* ain't he a lazy person ??*/		

   	  if(ld.username->text().isEmpty())
   	  		ld.username->setText("/");
			
   	  if(t=ConnectToOracle((const char*)ld.username->text()
   	  			  ,(const char*)ld.passwd->text()
   	  			  ,(const char*)ld.database->text()
   	  			  ,&err))
   	  {

   	  		this->username=ld.username->text();
   	  		this->password=ld.passwd->text();
   	  		this->database=ld.database->text();
   	  		this->SetSchemaName();
	
   	  		emit TreeChanged();
   	   				
   	  		/* set the connected attribute */
   	  		this->connected=TRUE;
   	   			
   	  	keeptrying = FALSE;
			}
			else
			{
			 		QMessageBox::message("Error",err);
					DisconnectFromOracle();
			 		keeptrying = TRUE;
			 		free(err);

			}
		}
		else
		{
			keeptrying = FALSE;
		}
	}while(keeptrying);

	this->SetNewConnectString();
	
	/* reinstall the zombie-killer */
	signal(SIGCHLD,Signal);
}


void ApplicationWindow::InsertDebugText(QString debugstring)
{
	if(this->debugstate)
		this->debugdialog->InsertDebugText(QString(debugstring));
}

void ApplicationWindow::InsertTabmanText(QString tabmanstring)
{
	if(this->tabmanscanstate)
		this->tabmandialog->InsertTabmanText(QString(tabmanstring));
}

void ApplicationWindow::ScanTabmanPipe()
{ 
	char message[32000];
	char message_type[4000];
	char *err=NULL;
	int return_value;
	
	if(this->connected)
	{
		while((return_value=ReadFromPipe("TABMANPIPE",message_type,message,&err))==0)
		{				
			this->InsertTabmanText(QString(message));
			if(!strcmp(message_type,"END"))
			{
				emit ListChanged();
				emit TreeChanged();
				Debug("Refresh done");
			}
			this->tabmandialog->Blink();
		}
		
		if(return_value==-1) /*error while scanning the pipe*/
		{
				sprintf(message,"Error scanning pipe : %s",err);
				this->InsertTabmanText(QString(message));
		}
		this->tabmandialog->Blink();		
	}
}


void ApplicationWindow::ShowAbout()
{		
	AboutDialog about(this,"About");
	about.exec();
}

void ApplicationWindow::SetNewConnectString()
{		
	QString db;

	if(!this->connected)
		this->statusbar->ShowConnectString("NOT Connected");
	else
	{
   		if (this->database.isEmpty())
   			db = QString(getenv("TWO_TASK"));
   		else
   			db = this->database;
   		   				
		if (db.isEmpty())
   			db = QString("LOCAL");
   		   					
		this->statusbar->ShowConnectString("Connected as : "+this->username+"@"+db);
	}
}

void ApplicationWindow::SetSchemaName()
{
	char schema[31];
	char *err;
	
	if(!GetSchemaName(schema,&err))
	{
		QMessageBox::message("Error",err);
	}
	else
		this->schemaname = QString(schema);
}

void ApplicationWindow::show()
{
	//tabmandialog->show();
	QMainWindow::show();
}


void ApplicationWindow::StartSqlPlus()
{
	QString sysstring;
	char sstr[500];
	
	sysstring = "xterm -e sqlplus "+this->username;
	if(!this->password.isEmpty() )
		sysstring = sysstring + '/' + this->password;
	if(!this->database.isEmpty() )
		sysstring = sysstring + "@" + this->database;
	
	strcpy(sstr ,sysstring);
	if( fork()==0)
	{
		system(sstr);
		exit(0);
	}	
}

void ApplicationWindow::DoFullRefresh()
{
	emit TreeChanged();
	emit ListChanged();
}

void ApplicationWindow::TreeClicked()
{
	/* deselect the listview item (if any)*/
}

void ApplicationWindow::ListClicked()
{
	/* Deselect the tree */
	//this->schematree->setSelected((SchemaListItem*)this->schematree->currentItem(),FALSE);
}

void ApplicationWindow::NullSlot()
{
	/* This method does nothing (as it's name sugests) */
}

void ApplicationWindow::ShowObjectProperties()
{
	if ( !strcmp(this->itemtype,"TABLE") )
		this->ShowTableDialog();
}

void ApplicationWindow::SetNewObjectSelection(QString iname,QString itype,
											  QString iparentname,QString iparenttype)
{
	Debug("------------  begin new selection  -----------------");
	Debug("Item name        = '%s'",(const char *)iname);
	Debug("Item type        = '%s'",(const char *)itype);
	Debug("Item parent name = '%s'",(const char *)iparentname);
	Debug("Item parent type = '%s'",(const char *)iparenttype);
	Debug("-------------  end new selection  ------------------");
	
	if (!strcmp(itype,"TABLE"))
	{
		this->itemname       = iname;
		this->itemtype       = itype;
		this->itemparentname = iparentname;
		this->itemparenttype = iparenttype;
		this->tabledialog->SetTableName(iname,this->schemaname);
	}
}